# Load the data
hght_wide<-read.table('C:/GRE2016/Data/bgs_height_wide.dat', na.string='.')

# Give the variable names
names(hght_wide)<-c('id', 'hght01', 'hght03', 'hght06', 'hght09', 'hght12', 
                          'hght15', 'hght18', 'hght24', 'hght36')

library(OpenMx)

################# Jenss-Bayley Growth Model ################# 
jb.hght.omx <- mxModel("Jenss Bayley Growth Model, Path Specification", type="RAM",
mxData(observed=hght_wide, type="raw"),
manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
latentVars=c('eta_1','eta_2','eta_3'),

# Residual Variances
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values =10, labels='theta'),

# Latent Variable Covariances
mxPath(from=c('eta_1','eta_2','eta_3'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(1,0,0,1,0,1), 
 labels=c('psi_11','psi_21','psi_31','psi_22','psi_32','psi_33')),

# Factor Loadings
mxPath(from='eta_1', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='eta_3', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]', 'Load[2,1]', 'Load[3,1]', 'Load[4,1]', 'Load[5,1]', 'Load[6,1]', 'Load[7,1]', 'Load[8,1]', 'Load[9,1]')),

# Latent Variable Means
mxPath(from='one', to=c('eta_1','eta_2','eta_3'),
 arrows=1, free=TRUE, values=c(80, 15, -1), labels=c('alpha_1','alpha_2','alpha_3')),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),

mxAlgebra(exp(gamma * time)-one, name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit)

#free parameters:
#      name matrix    row    col    Estimate  Std.Error lbound ubound
#1    theta      S hght01 hght01   0.6709180 0.05002748              
#2   psi_11      S  eta_1  eta_1   7.3887546 1.35187369              
#3   psi_21      S  eta_1  eta_2   0.6152326 0.37492526              
#4   psi_22      S  eta_2  eta_2   0.7396310 0.18121408              
#5   psi_31      S  eta_1  eta_3   4.8585302 1.42019058              
#6   psi_32      S  eta_2  eta_3   0.6706268 0.48959317              
#7   psi_33      S  eta_3  eta_3   9.4222170 2.14660656              
#8  alpha_1      M      1  eta_1  51.0933104 0.35235328              
#9  alpha_2      M      1  eta_2   9.2744269 0.17229025              
#10 alpha_3      M      1  eta_3 -17.8806580 0.46354837              
#11       g  gamma      1      1  -2.0629718 0.07802196              
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#saturated -2 log likelihood:  NA 
#number of observations:  83 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:   838.0950           2000.095                   NA
#BIC:  -540.6442           2026.702             1992.006
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-09-19 12:58:47 
#frontend time: 0.6470652 secs 
#backend time: 0.2490249 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.89609 secs 
#cpu time: 0.89609 secs 
#OpenMx version number: 999.0.0-3473 


jb.hght.fit$Load

#mxAlgebra 'Load' 
#@formula:  exp(gamma * time) - one 
#@result:
#            [,1]
# [1,] -0.1578909
# [2,] -0.4029433
# [3,] -0.6435233
# [4,] -0.7871632
# [5,] -0.8729244
# [6,] -0.9241287
# [7,] -0.9547005
# [8,] -0.9838518
# [9,] -0.9979480
#dimnames: NULL


################# Latent Basis Growth Model ################# 

latent.hght.omx <- mxModel("Latent Basis Growth Model, Path Specification", type="RAM",
mxData(observed=hght_wide, type="raw"),
manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
latentVars=c('eta_1','eta_2'),

# Residual Variances
mxPath( from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=10, labels='theta'),

# Latent Variable Covariances
mxPath(from=c('eta_1','eta_2'), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c('psi_11','psi_21','psi_22')),

# Factor Loadings
mxPath(from='eta_1', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=c(FALSE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,FALSE), values=c(0, .2, .3, .4, .5, .6, .7, .8, 1), labels=c(NA,'L22','L32','L42','L52','L62','L72','L82',NA)),

# Latent Variable Means
mxPath(from='one', to=c('eta_1','eta_2'),
 arrows=1, free=TRUE, values=c(50, 45), labels=c('alpha_1','alpha_2'))

) # Close Model 

latent.hght.fit <- mxRun(latent.hght.omx)

summary(latent.hght.fit)


#free parameters:
#      name matrix    row    col   Estimate   Std.Error lbound ubound
#1      L22      A hght03  eta_2  0.1470678 0.003537664              
#2      L32      A hght06  eta_2  0.3088311 0.003425829              
#3      L42      A hght09  eta_2  0.4095697 0.003392012              
#4      L52      A hght12  eta_2  0.5029529 0.003350233              
#5      L62      A hght15  eta_2  0.5870968 0.003550548              
#6      L72      A hght18  eta_2  0.6568478 0.003552048              
#7      L82      A hght24  eta_2  0.7832834 0.003641129              
#8    theta      S hght01 hght01  0.8157837 0.056355496              
#9   psi_11      S  eta_1  eta_1  5.1385341 0.888383364              
#10  psi_21      S  eta_1  eta_2 -0.9284709 0.971928318              
#11  psi_22      S  eta_2  eta_2 10.2311739 1.980208052              
#12 alpha_1      M      1  eta_1 54.5425931 0.277776692              
#13 alpha_2      M      1  eta_2 42.1000334 0.408747958              
#
#observed statistics:  581 
#estimated parameters:  13 
#degrees of freedom:  568 
#-2 log likelihood:  1996.204 
#saturated -2 log likelihood:  NA 
#number of observations:  83 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:   860.2045           2022.204                   NA
#BIC:  -513.6970           2053.649             2012.644
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-09-19 13:02:51 
#frontend time: 0.02900314 secs 
#backend time: 0.1090109 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.1380141 secs 
#cpu time: 0.1380141 secs 
#OpenMx version number: 999.0.0-3473 


################# Spline Growth Model with Estimated Knot Point ################# 

spl.hght.omx <- mxModel("Spline Growth Model, Path Specification", type="RAM", 
mxData(observed=hght_wide, type="raw"),
manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'), latentVars=c('eta_1','eta_2','eta_3'),

# Residual Variances
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'), arrows=2, free=TRUE, values=10, labels='theta'),

# Latent Variable Covariances 
mxPath(from=c('eta_1','eta_2','eta_3'), connect='unique.pairs', arrows=2,
free=TRUE, values=c(1,0,0,1,0,1), labels=c('psi_11','psi_21','psi_31','psi_22','psi_32','psi_33')),

# Factor Loadings
mxPath(from='eta_1', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'), arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'), arrows=1, free=FALSE, values=0, labels=c('L11[1,1]', 'L21[1,1]', 'L31[1,1]', 'L41[1,1]', 'L51[1,1]', 'L61[1,1]', 'L71[1,1]', 'L81[1,1]', 'L91[1,1]')),

mxPath(from='eta_3', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'), arrows=1, free=FALSE, values=0, labels=c('L12[1,1]', 'L22[1,1]', 'L32[1,1]', 'L42[1,1]', 'L52[1,1]', 'L62[1,1]', 'L72[1,1]', 'L82[1,1]', 'L92[1,1]')),

# Latent Variable Means  
mxPath(from='one', to=c('eta_1','eta_2','eta_3'), arrows=1, free=TRUE, values=c(70, 3, 1), labels=c('alpha_1','alpha_2','alpha_3')),

# Additional Parameter and Constraints
mxMatrix('Full', 1, 1, free=TRUE, values=7, labels='g', name='gamma'),

mxAlgebra(min(0,  1-gamma), name='L11'),
mxAlgebra(min(0,  3-gamma), name='L21'),
mxAlgebra(min(0,  6-gamma), name='L31'),
mxAlgebra(min(0,  9-gamma), name='L41'),
mxAlgebra(min(0, 12-gamma), name='L51'),
mxAlgebra(min(0, 15-gamma), name='L61'),
mxAlgebra(min(0, 18-gamma), name='L71'),
mxAlgebra(min(0, 24-gamma), name='L81'),
mxAlgebra(min(0, 36-gamma), name='L91'),

mxAlgebra(max(0,  1-gamma), name='L12'),
mxAlgebra(max(0,  3-gamma), name='L22'),
mxAlgebra(max(0,  6-gamma), name='L32'),
mxAlgebra(max(0,  9-gamma), name='L42'),
mxAlgebra(max(0, 12-gamma), name='L52'),
mxAlgebra(max(0, 15-gamma), name='L62'),
mxAlgebra(max(0, 18-gamma), name='L72'),
mxAlgebra(max(0, 24-gamma), name='L82'),
mxAlgebra(max(0, 36-gamma), name='L92')

) # Close Model 

spl.hght.fit <- mxRun(spl.hght.omx)

summary(spl.hght.fit)

#free parameters:
#      name matrix    row    col     Estimate   Std.Error lbound ubound
#1    theta      S hght01 hght01  1.475858661 0.110315026              
#2   psi_11      S  eta_1  eta_1  5.907146657 1.045698141              
#3   psi_21      S  eta_1  eta_2  0.265585607 0.112325672              
#4   psi_22      S  eta_2  eta_2  0.069665792 0.020472517              
#5   psi_31      S  eta_1  eta_3  0.075625276 0.024902494              
#6   psi_32      S  eta_2  eta_3  0.008191522 0.003366130              
#7   psi_33      S  eta_3  eta_3  0.003271379 0.001122327              
#8  alpha_1      M      1  eta_1 71.688102934 0.333014682              
#9  alpha_2      M      1  eta_2  2.565283804 0.052740943              
#10 alpha_3      M      1  eta_3  0.903088429 0.009925738              
#11       g  gamma      1      1  7.516692798 0.133048909              
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  2264.687 
#saturated -2 log likelihood:  NA 
#number of observations:  83 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:  1124.6872           2286.687                   NA
#BIC:  -254.0519           2313.294             2278.598
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-09-19 13:06:18 
#frontend time: 0.1440148 secs 
#backend time: 0.09700894 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.2410238 secs 
#cpu time: 0.2410238 secs 
#OpenMx version number: 999.0.0-3473 

