hght_long <- read.table('C:/GRE2016/Data/bgs_height_long.dat', na.string='.')

# Give the variable names
names(hght_long)<-c('id', 'age', 'hght')

summary(hght_long)
attach(hght_long)

library(nlme)

################### Jenss-Bayley Growth Model ################### 

hght.jb.nlme <- nlme(hght~b_1i+b_2i*(age/12)+b_3i*(exp(gamma*(age/12))-1),
                      data=hght_long,
                      fixed=b_1i+b_2i+b_3i+gamma~1,
                      random=b_1i+b_2i+b_3i~1,
                      groups=~id,
                      start=c(50, 10, -18, -2),
                      na.action=na.omit)
summary(hght.jb.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: hght ~ b_1i + b_2i * (age/12) + b_3i * (exp(gamma * (age/12)) -      1) 
# Data: hght_long 
#       AIC      BIC    logLik
#  2000.239 2048.252 -989.1197
#
#Random effects:
# Formula: list(b_1i ~ 1, b_2i ~ 1, b_3i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr       
#b_1i     2.7199531 b_1i  b_2i 
#b_2i     0.8590517 0.260      
#b_3i     3.0567734 0.583 0.248
#Residual 0.8193595            
#
#Fixed effects: b_1i + b_2i + b_3i + gamma ~ 1 
#          Value Std.Error  DF   t-value p-value
#b_1i   51.04753 0.3501220 495 145.79927       0
#b_2i    9.31413 0.1649162 495  56.47799       0
#b_3i  -17.81609 0.4537913 495 -39.26054       0
#gamma  -2.08950 0.0731175 495 -28.57738       0
# Correlation: 
#      b_1i   b_2i   b_3i  
#b_2i  -0.034              
#b_3i   0.412  0.543       
#gamma  0.350 -0.655 -0.392
#
#Standardized Within-Group Residuals:
#        Min          Q1         Med          Q3         Max 
#-3.62957188 -0.51594952 -0.02464763  0.52653168  2.87228860 
#
#Number of Observations: 581
#Number of Groups: 83 


################### Latent Basis Growth Model ################### 

for(i in 1:747){
       if (hght_long$age[i] == 1) {hght_long$age1[i]=1}
       if (hght_long$age[i] != 1) {hght_long$age1[i]=0}
       if (hght_long$age[i] == 3) {hght_long$age3[i]=1}
       if (hght_long$age[i] != 3) {hght_long$age3[i]=0}
       if (hght_long$age[i] == 6) {hght_long$age6[i]=1}
       if (hght_long$age[i] != 6) {hght_long$age6[i]=0}
       if (hght_long$age[i] == 9) {hght_long$age9[i]=1}
       if (hght_long$age[i] != 9) {hght_long$age9[i]=0}
       if (hght_long$age[i] == 12) {hght_long$age12[i]=1}
       if (hght_long$age[i] != 12) {hght_long$age12[i]=0}
       if (hght_long$age[i] == 15) {hght_long$age15[i]=1} 
       if (hght_long$age[i] != 15) {hght_long$age15[i]=0} 
       if (hght_long$age[i] == 18) {hght_long$age18[i]=1} 
       if (hght_long$age[i] != 18) {hght_long$age18[i]=0}
       if (hght_long$age[i] == 24) {hght_long$age24[i]=1}
       if (hght_long$age[i] != 24) {hght_long$age24[i]=0}
       if (hght_long$age[i] == 36) {hght_long$age36[i]=1}
       if (hght_long$age[i] != 36) {hght_long$age36[i]=0}
       }

hght.latent.nlme <- nlme(hght~age1 *(b_1i+b_2i*0)+
                              age3 *(b_1i+b_2i*A_2)+
                              age6 *(b_1i+b_2i*A_3)+
                              age9 *(b_1i+b_2i*A_4)+
                              age12*(b_1i+b_2i*A_5)+
                              age15*(b_1i+b_2i*A_6)+
                              age18*(b_1i+b_2i*A_7)+
                              age24*(b_1i+b_2i*A_8)+
                              age36*(b_1i+b_2i*1),
                         data=hght_long,
                         fixed=b_1i+b_2i+A_2+A_3+A_4+A_5+A_6+A_7+A_8~1,
                         random=b_1i+b_2i~1,
                         groups=~id,
                         start=c(60, 20, .3, .4, .5, .6, .7, .8, .9),
                         na.action=na.omit) 

summary(hght.latent.nlme)



#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: hght ~ age1 * (b_1i + b_2i * 0) + age3 * (b_1i + b_2i * A_2) +      age6 * (b_1i + b_2i * A_3) + age9 * (b_1i + b_2i * A_4) +      age12 * (b_1i + b_2i * A_5) + age15 * (b_1i + b_2i * A_6) +      age18 * (b_1i + b_2i * A_7) + age24 * (b_1i + b_2i * A_8) +      age36 * (b_1i + b_2i * 1) 
# Data: hght_long 
#       AIC      BIC   logLik
#  2022.224 2078.966 -998.112
#
#Random effects:
# Formula: list(b_1i ~ 1, b_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr  
#b_1i     2.2669367 b_1i  
#b_2i     3.1982959 -0.128
#Residual 0.9031948       
#
#Fixed effects: b_1i + b_2i + A_2 + A_3 + A_4 + A_5 + A_6 + A_7 + A_8 ~ 1 
#        Value Std.Error  DF   t-value p-value
#b_1i 54.54834 0.2797289 490 195.00427       0
#b_2i 42.08689 0.4113588 490 102.31187       0
#A_2   0.14678 0.0035631 490  41.19609       0
#A_3   0.30874 0.0034520 490  89.43782       0
#A_4   0.40956 0.0034124 490 120.02170       0
#A_5   0.50299 0.0033746 490 149.05122       0
#A_6   0.58723 0.0035781 490 164.11636       0
#A_7   0.65704 0.0035792 490 183.57248       0
#A_8   0.78358 0.0036713 490 213.43595       0
# Correlation: 
#     b_1i   b_2i   A_2    A_3    A_4    A_5    A_6    A_7   
#b_2i -0.239                                                 
#A_2  -0.290  0.148                                          
#A_3  -0.244  0.061  0.429                                   
#A_4  -0.215  0.021  0.386  0.398                            
#A_5  -0.183 -0.005  0.338  0.358  0.365                     
#A_6  -0.140 -0.047  0.282  0.318  0.338  0.346              
#A_7  -0.119 -0.076  0.248  0.293  0.321  0.345  0.341       
#A_8  -0.076 -0.122  0.189  0.253  0.291  0.329  0.340  0.365
#
#Standardized Within-Group Residuals:
#        Min          Q1         Med          Q3         Max 
#-3.58052647 -0.53575511  0.02044748  0.52029420  4.35506540 
#
#Number of Observations: 581
#Number of Groups: 83 


################### Spline  Growth Model ################### 

hght.spline.nlme <- nlme(hght~b_1i+b_2i*(pmin(0,age-gamma))+b_3i*(pmax(0,age-gamma)),
                      data=hght_long,
                      fixed=b_1i+b_2i+b_3i+gamma~1,
                      random=b_1i+b_2i+b_3i~1,
                      groups=~id,
                      start=c(60, 5, 2, 8),
                      na.action=na.omit)
summary(hght.spline.nlme) 

#Random effects:
# Formula: list(b_1i ~ 1, b_2i ~ 1, b_3i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev     Corr       
#b_1i     2.43555766 b_1i  b_2i 
#b_2i     0.26036626 0.418      
#b_3i     0.05674639 0.548 0.550
#Residual 1.21623023            
#
#Fixed effects: b_1i + b_2i + b_3i + gamma ~ 1 
#         Value Std.Error  DF   t-value p-value
#b_1i  71.82716 0.3353841 495 214.16390       0
#b_2i   2.53762 0.0525520 495  48.28783       0
#b_3i   0.90117 0.0098949 495  91.07447       0
#gamma  7.62649 0.1372391 495  55.57085       0
# Correlation: 
#      b_1i   b_2i   b_3i  
#b_2i  -0.040              
#b_3i   0.041  0.199       
#gamma  0.514 -0.650 -0.234
#
#Standardized Within-Group Residuals:
#        Min          Q1         Med          Q3         Max 
#-2.64758396 -0.55058440  0.01787728  0.58480214  2.59728429 
#
#Number of Observations: 581
#Number of Groups: 83 


 